@@ -49,7 +49,7 @@
 #include "eog-wrap-list.h"
 #include "eog-scroll-view.h"
 #include "eog-collection-view.h"
-#include "eog-collection-model.h"
+#include "eog-image-list.h"
 #include "eog-collection-marshal.h"
 #include "eog-full-screen.h"
 #include "eog-info-view.h"
@@ -84,7 +84,7 @@ static const gchar *pref_key[] = {
 };
 
 struct _EogCollectionViewPrivate {
-	EogCollectionModel      *model;
+	EogImageList            *model;
 
 	GtkWidget               *wraplist;
 	GtkWidget               *scroll_view;
@@ -119,6 +119,17 @@ static guint eog_collection_view_signals [LAST_SIGNAL];
 BONOBO_CLASS_BOILERPLATE (EogCollectionView, eog_collection_view,
 			  BonoboPersistFile, BONOBO_TYPE_PERSIST_FILE);
 
+static void
+free_image_list (GList *list)
+{
+	GList *it;
+
+	for (it = list; it != NULL; it = it->next) {
+		g_object_unref (G_OBJECT (it->data));
+	}
+	g_list_free (list);
+}
+
 
 static void
 verb_SlideShow_cb (BonoboUIComponent *uic, 
@@ -127,26 +138,27 @@ verb_SlideShow_cb (BonoboUIComponent *uic,
 {
 	EogCollectionView *view;
 	GtkWidget *show;
-	GList *images = NULL;
 	EogImage *start_image = NULL;
-	gboolean free_list = FALSE;
+	EogImageList *list;
 
 	view = EOG_COLLECTION_VIEW (user_data);
 
 	if (eog_wrap_list_get_n_selected (EOG_WRAP_LIST (view->priv->wraplist)) > 1) {
-		images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));
-		free_list = TRUE;
+		GList *image_list;
+
+		image_list = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));
+		list = eog_image_list_new_from_glist (image_list);
+
+		free_image_list (image_list);
 	}
 	else {
-		images = eog_collection_model_get_image_list (view->priv->model);
+		list = g_object_ref (view->priv->model);
 	}
 	start_image = eog_wrap_list_get_first_selected_image (EOG_WRAP_LIST (view->priv->wraplist));
 
-	show = eog_full_screen_new (images, start_image);
+	show = eog_full_screen_new (list, start_image);
 	g_object_unref (start_image);
-
-	if (free_list)
-		g_list_free (images);
+	g_object_unref (list);
 
 	gtk_widget_show (show);
 }
@@ -190,6 +202,7 @@ verb_Undo_cb (BonoboUIComponent *uic, gpointer data, const char *name)
 		EogImage *image = EOG_IMAGE (it->data);
 		eog_image_undo (image);
 	}
+	free_image_list (images);
 }
 
 /*
@@ -262,15 +275,15 @@ save_image_loading_finished (EogImage *image, gpointer data)
 	g_signal_handler_disconnect (G_OBJECT (image), ctx->failed_id);
 
 	if (eog_image_is_modified (image) && !ctx->cancel) {
-			GError *error = NULL;
-			GnomeVFSURI *uri;
-			
-			uri = eog_image_get_uri (image);
-			
-			if (!eog_image_save (image, uri, &error)) {
-				/* FIXME: indicate failed state to the user */
-			}
-			gnome_vfs_uri_unref (uri);
+		GError *error = NULL;
+		GnomeVFSURI *uri;
+		
+		uri = eog_image_get_uri (image);
+		
+		if (!eog_image_save (image, uri, &error)) {
+			/* FIXME: indicate failed state to the user */
+		}
+		gnome_vfs_uri_unref (uri);
 	}
 
 	if (image != ctx->view->priv->displayed_image) {
@@ -306,8 +319,6 @@ save_image_init_loading (SaveContext *ctx)
 		ctx->failed_id = g_signal_connect (image, "loading_failed", G_CALLBACK (save_image_loading_failed), ctx);
 		ctx->image_list = g_list_delete_link (ctx->image_list, ctx->image_list);
 
-		g_object_ref (image);
-
 		eog_image_load (image, EOG_IMAGE_LOAD_COMPLETE);
 	}
 
@@ -350,7 +361,7 @@ save_dlg_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_OK:
 		view = ctx->view;
 		if (ctx->image_list != NULL) {
-			g_list_free (ctx->image_list);
+			free_image_list (ctx->image_list);
 		}
 		g_free (ctx);
 		g_timeout_add (1250, save_dlg_close, dialog);
@@ -471,7 +482,8 @@ apply_transformation (EogCollectionView *view, EogTransform *trans)
 		g_signal_handler_disconnect (image, id);
 		data->counter++;
 	}
-
+	
+	free_image_list (images);
 	g_free (data);
 	g_object_unref (trans);
 }
@@ -607,6 +619,12 @@ eog_collection_view_create_ui (EogCollectionView *view)
 						     collection_verbs,
 						     view);
 
+	/* special case for SUN keyboards where F11 key has different keysym */
+	if (eog_full_screen_enable_SunF36()) {
+		bonobo_ui_component_add_verb (priv->uic, "FullScreenHiddenSun",
+					      verb_SlideShow_cb, view);
+	}
+
 	if (!priv->has_zoomable_frame) {
 		bonobo_ui_util_set_ui (priv->uic, DATADIR, "eog-image-view-ctrl-ui.xml", "EogCollectionView", NULL);
 		
@@ -797,7 +815,8 @@ eog_collection_view_get_prop (BonoboPropertyBag *bag,
 		gchar *base_uri;
 		gchar *title;
 
-		base_uri = eog_collection_model_get_base_uri (priv->model);
+		/* FIXME: what should happen with this? */
+		base_uri = NULL; /*  eog_collection_model_get_base_uri (priv->model); */
 
 		if (base_uri == NULL)
 			title = g_strdup (_("Collection View"));
@@ -813,7 +832,7 @@ eog_collection_view_get_prop (BonoboPropertyBag *bag,
 		gchar *str;
 		gint nimg, nsel;
 			
-		nimg = eog_collection_model_get_length (priv->model);
+		nimg = eog_image_list_length (priv->model);
 		nsel = eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist));
 
 		str = g_new0 (guchar, 70);
@@ -914,7 +933,6 @@ handle_selection_changed (EogWrapList *list, EogCollectionView *view)
 		if (priv->displayed_image != NULL) {
 			g_signal_handler_disconnect (priv->displayed_image, priv->progress_handler_id);
 
-			eog_image_free_mem (priv->displayed_image);
 			g_object_unref (priv->displayed_image);
 			priv->displayed_image = NULL;
 		}
@@ -932,7 +950,7 @@ handle_selection_changed (EogWrapList *list, EogCollectionView *view)
 }
 
 static void
-model_size_changed (EogCollectionModel *model, GList *id_list,  gpointer data)
+model_size_changed (EogImageList *model, GList *id_list,  gpointer data)
 {
 	EogCollectionView *view;
 
@@ -945,7 +963,7 @@ model_size_changed (EogCollectionModel *model, GList *id_list,  gpointer data)
 
 
 static void
-model_base_uri_changed (EogCollectionModel *model, gpointer data)
+model_base_uri_changed (EogImageList *model, gpointer data)
 {
 	EogCollectionView *view;
 
@@ -1072,7 +1090,7 @@ load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 	if (text_uri == CORBA_OBJECT_NIL) return 0;
 
 	if (priv->model == NULL) {
-		priv->model = eog_collection_model_new ();
+		priv->model = eog_image_list_new ();
 		eog_wrap_list_set_model (EOG_WRAP_LIST (priv->wraplist), priv->model);
 
 		/* construct widget */
@@ -1093,7 +1111,7 @@ load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 
 	}
 
-	eog_collection_model_add_uri (priv->model, (gchar*)text_uri); 
+	eog_image_list_add_directory (priv->model, (gchar*)text_uri); 
 
 	return 0;
 }
