@@ -22,6 +22,7 @@
 #include <math.h>
 #include <string.h>
 #include <glib/gmain.h>
+#include <glib/gmacros.h>
 #include <libgnome/gnome-macros.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkdnd.h>
@@ -33,8 +34,13 @@
 
 #define COLLECTION_DEBUG 0
 
+#define EOG_WRAP_LIST_BORDER  12
+static const int MAX_ITEM_WIDTH = EOG_COLLECTION_ITEM_MAX_WIDTH;
+
 #define EOG_DND_URI_LIST_TYPE 	         "text/uri-list"
 
+
+
 enum {
 	EOG_DND_URI_LIST
 };
@@ -62,10 +68,10 @@ enum {
 static guint eog_wrap_list_signals [LAST_SIGNAL];
 
 enum {
-	GLOBAL_SIZE_CHANGED,
+	GLOBAL_WIDGET_SIZE_CHANGED,
 	GLOBAL_SPACING_CHANGED,
-	GLOBAL_LAYOUT_MODE_CHANGED,
-	GLOBAL_HINT_LAST
+	ITEM_SIZE_CHANGED,
+	UPDATE_HINT_LAST
 };
 
 typedef struct {
@@ -88,7 +94,7 @@ struct _EogWrapListPrivate {
 	int n_selected_items;
 	
 	/* Model to use. */
-	EogCollectionModel *model;
+	EogImageList *model;
 
 	/* Signal connection IDs for the models */
 	int model_ids[MODEL_SIGNAL_LAST];
@@ -107,9 +113,6 @@ struct _EogWrapListPrivate {
 	/* Number of items */
 	guint n_items; 
 
-	/* Layout mode to use for row/col calculating. */
-	EogLayoutMode lm;
-
 	/* Number of rows. */
 	guint n_rows;
 
@@ -120,7 +123,7 @@ struct _EogWrapListPrivate {
 	gint idle_handler_id;
 
 	/* Update hints */
-	gboolean global_update_hints [GLOBAL_HINT_LAST];
+	gboolean global_update_hints [UPDATE_HINT_LAST];
 
 	/* last id of thumbnail the user clicked */
 	GnomeCanvasItem *last_item_clicked;
@@ -218,7 +221,6 @@ eog_wrap_list_instance_init (EogWrapList *wlist)
 	priv->col_spacing = 0;
 	priv->idle_handler_id = -1;
 	priv->n_items = 0;
-	priv->lm = EOG_LAYOUT_MODE_VERTICAL;
 	priv->n_rows = 0;
 	priv->n_cols = 0;
 	priv->last_item_clicked = NULL;
@@ -307,14 +309,12 @@ static gint
 get_item_view_position (EogWrapList *wlist, GnomeCanvasItem *item)
 {
 	EogWrapListPrivate *priv;
-	double x1, y1, x2, y2;
-	gint row, col, n;
+	guint row, col, n;
 	
 	priv = wlist->priv;
-	gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
 
-	col = (x1 - priv->col_spacing) / (priv->item_width + priv->col_spacing);
-	row = (y1 - priv->row_spacing) / (priv->item_height + priv->row_spacing);
+	col = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (item), "column"));
+	row = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (item), "row"));
 
 	n = col + priv->n_cols * row;
 
@@ -455,7 +455,7 @@ static gint
 handle_item_event (GnomeCanvasItem *item, GdkEvent *event,  EogWrapList *wlist) 
 {
 	EogWrapListPrivate *priv;
-	EogCollectionModel *model;
+	EogImageList *model;
 	EogCollectionItem *eci;
 	GQuark id = 0;
 	gboolean ret_val = TRUE;
@@ -570,6 +570,18 @@ handle_item_event (GnomeCanvasItem *item, GdkEvent *event,  EogWrapList *wlist)
 	return ret_val;
 }
 
+static void
+handle_item_size_changed (GnomeCanvasItem *item, EogWrapList *wlist)
+{
+	EogWrapListPrivate *priv;
+
+	priv = wlist->priv;
+
+	priv->global_update_hints [ITEM_SIZE_CHANGED] = TRUE;
+
+	request_update (wlist);
+}
+
 /* Size_allocate handler for the abstract wrapped list view */
 static void
 eog_wrap_list_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
@@ -582,7 +594,7 @@ eog_wrap_list_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
 	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
 
-	priv->global_update_hints[GLOBAL_SIZE_CHANGED] = TRUE;
+	priv->global_update_hints[GLOBAL_WIDGET_SIZE_CHANGED] = TRUE;
 
 	request_update (wlist);
 }       
@@ -795,35 +807,35 @@ static void eog_wrap_list_drag_data_get_cb (GtkWidget *widget,
 	default:
 		g_assert_not_reached ();
 	}
-
+	
 }
 
 
-static gboolean
-add_image (EogCollectionModel *model, EogImage *image, gpointer data)
+static void
+add_image (EogWrapList *wlist, EogImage *image)
 {
-	EogWrapList *wlist;
 	EogWrapListPrivate *priv;
 	GnomeCanvasItem *item;
 	
-	g_return_val_if_fail (EOG_IS_WRAP_LIST (data), FALSE);
+	g_return_if_fail (EOG_IS_WRAP_LIST (wlist));
 
-	wlist = EOG_WRAP_LIST (data);
 	priv = wlist->priv;
 	
 	item = eog_collection_item_new (gnome_canvas_root (GNOME_CANVAS (wlist)), image);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (handle_item_event), wlist);
+	g_signal_connect (G_OBJECT (item), "size_changed", G_CALLBACK (handle_item_size_changed), wlist);
 
 	priv->view_order = g_list_prepend (priv->view_order, item);
-
-	return TRUE;
 }
 
 static void
-model_prepared (EogCollectionModel *model, gpointer data)
+model_prepared (EogImageList *model, gpointer data)
 {
 	EogWrapList *wlist;
 	EogWrapListPrivate *priv;
+	EogIter *iter;
+	gboolean success;
+	EogImage *image;
 	GList *it;
 	
 	g_return_if_fail (EOG_IS_WRAP_LIST (data));
@@ -833,12 +845,19 @@ model_prepared (EogCollectionModel *model, gpointer data)
 	
 	g_assert (priv->view_order == NULL);
 
-	eog_collection_model_foreach (model, add_image, EOG_WRAP_LIST (data));
+	iter = eog_image_list_get_first_iter (model);
+	success = (iter != NULL);
+	for (; success; success = eog_image_list_iter_next (model, iter, FALSE)) {
+		image = eog_image_list_get_img_by_iter (model, iter);
+		add_image (wlist, image);
+		g_object_unref (image);
+	}
+	g_free (iter);
 
 	priv->view_order = g_list_reverse (priv->view_order);
 	priv->n_items = g_list_length (priv->view_order);
 
-	priv->global_update_hints[GLOBAL_SIZE_CHANGED] = TRUE;
+	priv->global_update_hints[GLOBAL_WIDGET_SIZE_CHANGED] = TRUE;
 	request_update (wlist);
 	
 	for (it = priv->view_order; it != NULL; it = it->next) {
@@ -856,7 +875,7 @@ model_prepared (EogCollectionModel *model, gpointer data)
 
 /* Handler for the interval_added signal from models */
 static void
-model_image_added (EogCollectionModel *model, EogImage *image, int position, gpointer data)
+model_image_added (EogImageList *model, EogImage *image, int position, gpointer data)
 {
 	GnomeCanvasItem *item;
 	EogWrapList *wlist;
@@ -874,7 +893,7 @@ model_image_added (EogCollectionModel *model, EogImage *image, int position, gpo
 	priv->view_order = g_list_insert (priv->view_order, item, position);
 	priv->n_items = g_list_length (priv->view_order);
 
-	priv->global_update_hints[GLOBAL_SIZE_CHANGED] = TRUE;
+	priv->global_update_hints[GLOBAL_WIDGET_SIZE_CHANGED] = TRUE;
 	request_update (wlist);
 	
 	eog_collection_item_load (EOG_COLLECTION_ITEM (item));
@@ -882,7 +901,7 @@ model_image_added (EogCollectionModel *model, EogImage *image, int position, gpo
 
 /* Handler for the interval_changed signal from models */
 static void
-model_image_removed (EogCollectionModel *model, GQuark id, gpointer data)
+model_image_removed (EogImageList *model, GQuark id, gpointer data)
 {
 #if 0
 	g_return_if_fail (EOG_IS_WRAP_LIST (data));
@@ -897,7 +916,7 @@ model_image_removed (EogCollectionModel *model, GQuark id, gpointer data)
 
 /* Set model handler for the wrapped list view */
 void
-eog_wrap_list_set_model (EogWrapList *wlist, EogCollectionModel *model)
+eog_wrap_list_set_model (EogWrapList *wlist, EogImageList *model)
 {
 	EogWrapListPrivate *priv;
 	int i;
@@ -920,7 +939,7 @@ eog_wrap_list_set_model (EogWrapList *wlist, EogCollectionModel *model)
 		g_object_ref (G_OBJECT (model));
 
 		priv->model_ids[MODEL_SIGNAL_PREPARED] = g_signal_connect (
-			G_OBJECT (model), "prepared",
+			G_OBJECT (model), "list-prepared",
 			G_CALLBACK (model_prepared),
 			wlist);
 
@@ -938,23 +957,6 @@ eog_wrap_list_set_model (EogWrapList *wlist, EogCollectionModel *model)
 	request_update (wlist);
 }
 
-void 
-eog_wrap_list_set_layout_mode (EogWrapList *wlist, EogLayoutMode lm)
-{
-	EogWrapListPrivate *priv;
-
-	g_return_if_fail (wlist != NULL);
-	g_return_if_fail (EOG_IS_WRAP_LIST (wlist));
-
-	priv = wlist->priv;
-
-	if (lm == priv->lm) return;
-
-	priv->lm = lm;
-	priv->global_update_hints[GLOBAL_LAYOUT_MODE_CHANGED] = TRUE;
-
-	request_update (wlist);
-}
 
 /**
  * eog_wrap_list_set_row_spacing:
@@ -1034,53 +1036,22 @@ do_layout_check (EogWrapList *wlist)
 	g_message ("do_layout_check called");
 #endif
 
-	if (priv->item_width == -1 || priv->item_height == -1) {
-		if (priv->view_order != NULL) {
-			double x1, y1, x2, y2;
-
-			gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (priv->view_order->data), &x1, &y1, &x2, &y2);
-			priv->item_width = (int) x2 - x1; 
-			priv->item_height = (int) y2 - y1;
-		}
-		else {
-			return FALSE;
-		}
-	}
-
+	if (priv->view_order == NULL)
+		return FALSE;
 
 	/* get canvas width */
 	cw = GTK_WIDGET (wlist)->allocation.width;
 	ch = GTK_WIDGET (wlist)->allocation.height;
 
 	/* calculate new number of  columns/rows */
-	switch (priv->lm) {
-	case EOG_LAYOUT_MODE_VERTICAL:
-		n_cols_new = cw / (priv->item_width + priv->col_spacing);
-		if (n_cols_new == 0)
-			n_cols_new = 1;
-		n_rows_new = (priv->n_items + n_cols_new - 1) / n_cols_new;
-		break;
-
-	case EOG_LAYOUT_MODE_HORIZONTAL:
-		n_rows_new = ch / (priv->item_height + priv->row_spacing);
-		if (n_rows_new == 0)
-			n_rows_new = 1;
-		n_cols_new = (priv->n_items + n_rows_new - 1) / n_rows_new;
-		break;
-
-	case EOG_LAYOUT_MODE_RECTANGLE:
-		n_rows_new = n_cols_new = sqrt (priv->n_items);
-		if (n_rows_new * n_cols_new < priv->n_items) {
-			if ((n_rows_new + 1) * n_cols_new < priv->n_items)
-				n_rows_new = n_cols_new = n_rows_new + 1;
-			else
-				n_rows_new = n_rows_new + 1;
-		}
-		break;
-
-	default:
-		g_assert_not_reached ();
-	}
+	n_cols_new = cw / (EOG_COLLECTION_ITEM_MAX_WIDTH + priv->col_spacing);
+	
+	if (cw > (n_cols_new * (EOG_COLLECTION_ITEM_MAX_WIDTH + priv->col_spacing) + EOG_COLLECTION_ITEM_MAX_WIDTH))
+		n_cols_new++;
+	
+	if (n_cols_new == 0)
+		n_cols_new = 1;
+	n_rows_new = (priv->n_items + n_cols_new - 1) / n_cols_new;
 
 #if COLLECTION_DEBUG	
 	g_print ("  ** canvas width: %i\n",cw);
@@ -1097,62 +1068,58 @@ do_layout_check (EogWrapList *wlist)
 	return TRUE;
 }
 
-static void 
-calculate_item_position (EogWrapList *wlist, 
-			 guint item_number,
-			 double *world_x,
-			 double *world_y)
-{
-	EogWrapListPrivate *priv;
-	guint row;
-	guint col;
-
-	priv = wlist->priv;
-
-	row = item_number / priv->n_cols;
-	col = item_number % priv->n_cols;
-		
-	*world_x = col * (priv->item_width + priv->col_spacing) + priv->col_spacing;
-	*world_y = row * (priv->item_height + priv->row_spacing) + priv->row_spacing;
-}
-
-typedef struct {
-	EogWrapList *wlist;
-	gint n;
-} RearrangeData; 
-
 static void
-rearrange_single_item (gpointer value, RearrangeData *data)
+set_item_position (EogCollectionItem *item, double x, double y)
 {
-	GnomeCanvasItem *item;
 	double x1, x2, y1, y2;
-	double x3, y3;
 	double xoff, yoff;
+
+	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (item), &x1, &y1, &x2, &y2);
 	
-	item = (GnomeCanvasItem*) value;
-	
-	gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
-	
-	calculate_item_position (data->wlist, data->n, &x3, &y3);
-	
-	xoff = x3-x1;
-	yoff = y3-y1;
+	xoff = x - x1;
+	yoff = y - y1;
 		
 	if (xoff || yoff)
-		gnome_canvas_item_move (item, xoff, yoff);
-	
-	data->n++;
+		gnome_canvas_item_move (GNOME_CANVAS_ITEM (item), xoff, yoff);
+}
+
+static void
+calculate_row_height (GList *it, int n_cols, 
+		      int *image_height, int *caption_height)
+{
+	int i;
+	EogCollectionItem *item;
+	int ih, ch, w;
+
+	*image_height = 0;
+	*caption_height = 0;
+
+	for (i = 0; (it != NULL) && (i < n_cols); i++, it = it->next) 
+	{
+		item = EOG_COLLECTION_ITEM (it->data);
+
+		eog_collection_item_get_size (item, &w, &ih, &ch);
+
+		*image_height = MAX (*image_height, ih);
+		*caption_height = MAX (*caption_height, ch);
+	}
 }
 
 static void
 do_item_rearrangement (EogWrapList *wlist)
 {
 	EogWrapListPrivate *priv;
-        RearrangeData data;
+	int max_image_height;
+	int max_caption_height;
+	int item_width;
+	int image_height;
+	int caption_height;
+	int row_offset;
+	int n, row, col;
+	double world_x, world_y;
 	double sr_width, sr_height;
-
-	data.wlist = wlist;
-	data.n = 0;
+	EogCollectionItem *item;
+	GList *it;
 
 #if COLLECTION_DEBUG
 	g_message ("do_item_rearrangement called");
@@ -1160,14 +1127,51 @@ do_item_rearrangement (EogWrapList *wlist)
 
 	priv = wlist->priv;
 
-	g_list_foreach (priv->view_order, 
-			(GFunc) rearrange_single_item, 
-			 &data);
-
+	row_offset = EOG_WRAP_LIST_BORDER - priv->row_spacing;
+	max_image_height = max_caption_height = 0;
+	it = priv->view_order;
+	col = 0;
+	
+	for (n = 0; it != NULL; it = it->next, n++) {
+		col = n % priv->n_cols;
+		row = n / priv->n_cols;
+		
+		if (col == 0) {
+			/* the offset for the new row is the
+			 * old offset plus the height of the
+			 * previous row 
+			 */
+			row_offset = row_offset + max_image_height + max_caption_height + 
+				priv->row_spacing;
+			
+			/* we start a new row and calculate
+			 * the row height/offset for the current row
+			 */
+			calculate_row_height (it, priv->n_cols,
+					      &max_image_height, &max_caption_height);
+		}
+		
+		item = EOG_COLLECTION_ITEM (it->data);
+		eog_collection_item_get_size (item, &item_width, &image_height, &caption_height);
+		item_width = MIN (item_width, MAX_ITEM_WIDTH); /* this is only to ensure 
+								  item_width <= MAX_ITEM_WIDTH */
+		
+		world_x = col * (MAX_ITEM_WIDTH + priv->col_spacing) + 
+			(MAX_ITEM_WIDTH - item_width) / 2;
+		world_y = row_offset + max_image_height - image_height;
+		
+		set_item_position (item, world_x, world_y);
+		
+		g_object_set_data (G_OBJECT (item), "row", GUINT_TO_POINTER (row));
+		g_object_set_data (G_OBJECT (item), "column", GUINT_TO_POINTER (col));
+	}
+	
+	
 	/* set new canvas scroll region */
-	sr_width =  priv->n_cols * (priv->item_width + priv->col_spacing) + priv->col_spacing;
-	sr_height = priv->n_rows * (priv->item_height + priv->row_spacing) + priv->row_spacing;
- 	gnome_canvas_set_scroll_region (GNOME_CANVAS (wlist), 
+	sr_width =  priv->n_cols * (MAX_ITEM_WIDTH + priv->col_spacing) - priv->col_spacing;
+	sr_height = row_offset + max_image_height + max_caption_height + EOG_WRAP_LIST_BORDER;
+	
+	gnome_canvas_set_scroll_region (GNOME_CANVAS (wlist), 
 					0.0, 0.0,
 					sr_width, sr_height);
 
@@ -1190,21 +1194,18 @@ do_update (EogWrapList *wlist)
 	
 	/* handle global updates */
 	if (priv->global_update_hints [GLOBAL_SPACING_CHANGED] ||
-	    priv->global_update_hints [GLOBAL_SIZE_CHANGED]) 
+	    priv->global_update_hints [GLOBAL_WIDGET_SIZE_CHANGED]) 
 	{
 		layout_check_needed = TRUE;
 		item_rearrangement_needed = TRUE;
 		
 		priv->global_update_hints[GLOBAL_SPACING_CHANGED] = FALSE;
-		priv->global_update_hints[GLOBAL_SIZE_CHANGED] = FALSE;
+		priv->global_update_hints[GLOBAL_WIDGET_SIZE_CHANGED] = FALSE;
 	} 
-	else if (priv->global_update_hints [GLOBAL_LAYOUT_MODE_CHANGED]) 
-	{
-		layout_check_needed = TRUE;
-
-		priv->global_update_hints[GLOBAL_LAYOUT_MODE_CHANGED] = FALSE;
+	else if (priv->global_update_hints [ITEM_SIZE_CHANGED]) {
+		item_rearrangement_needed = TRUE;
+		priv->global_update_hints [ITEM_SIZE_CHANGED] = FALSE;
 	}
-
 	
 	if (layout_check_needed && do_layout_check (wlist)) 
 		item_rearrangement_needed = TRUE;
