@@ -1,6 +1,9 @@
 #ifndef _EOG_IMAGE_PRIVATE_H_
 #define _EOG_IMAGE_PRIVATE_H_
 
+#if HAVE_EXIF
+#include <libexif/exif-data.h>
+#endif
 #include "eog-image.h"
 
 enum {
@@ -15,9 +18,17 @@ enum {
 	EOG_IMAGE_LOAD_STATUS_PROGRESS = 1 << 7
 };
 
+typedef enum {
+	EOG_IMAGE_STATUS_UNKNOWN,
+	EOG_IMAGE_STATUS_LOADING,
+	EOG_IMAGE_STATUS_LOADED,
+	EOG_IMAGE_STATUS_FAILED
+} EogImageStatus;
+ 
 struct _EogImagePrivate {
 	GnomeVFSURI *uri;
 	EogImageLoadMode mode;
+	EogImageStatus status;
 
 	GdkPixbuf *image;
 	GdkPixbuf *thumbnail;
@@ -57,4 +68,6 @@ struct _EogImagePrivate {
 };
 
 
+void eog_image_free_mem_private (EogImage *image);
+
 #endif /* _EOG_IMAGE_PRIVATE_H_ */
