@@ -43,6 +43,7 @@
 #include "eog-preferences.h"
 #include "libeog-marshal.h"
 #include "egg-recent.h"
+#include "eog-config-keys.h"
 
 /* Default size for windows */
 
@@ -84,6 +85,10 @@ struct _EogWindowPrivate {
 
 	EggRecentModel      *recent_model;
 	EggRecentViewBonobo *recent_view;
+
+	/* gconf notification ids */
+	guint view_statusbar_id;
+	guint view_toolbar_id;
 };
 
 enum {
@@ -101,6 +106,7 @@ static gint eog_window_delete (GtkWidget *widget, GdkEventAny *event);
 static gint eog_window_key_press (GtkWidget *widget, GdkEventKey *event);
 static void eog_window_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, 
 				    GtkSelectionData *selection_data, guint info, guint time);
+static void adapt_window_size (EogWindow *window);
 
 static BonoboWindowClass *parent_class;
 
@@ -269,7 +275,7 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 		about = gnome_about_new (
 			_("Eye of Gnome"),
 			VERSION,
-			_("Copyright (C) 2000-2002 The Free Software Foundation"),
+			"Copyright \xc2\xa9 2000-2003 Free Software Foundation, Inc.",
 			_("The GNOME image viewing and cataloging program."),
 			authors,
 			documenters,
@@ -320,6 +326,44 @@ verb_HelpContent_cb (BonoboUIComponent *uic, gpointer data, const char *cname)
 	}
 }
 
+static void
+verb_ShowHideAnyBar_cb (BonoboUIComponent           *ui_component,
+			const char                  *path,
+			Bonobo_UIComponent_EventType type,
+			const char                  *state,
+			gpointer                    *user_data)
+{
+	EogWindow *window;     
+	EogWindowPrivate *priv;
+	gboolean visible;
+	char *key = NULL;
+
+	g_return_if_fail(EOG_IS_WINDOW (user_data));
+
+	window = EOG_WINDOW (user_data);
+	priv = window->priv;
+
+	if (type != Bonobo_UIComponent_STATE_CHANGED) {
+		return;
+	}
+
+	if (state == NULL || path == NULL) {
+		return;
+	}
+
+	if (g_strcasecmp (path, "ShowHideToolbar") == 0) {
+		key = EOG_CONF_UI_TOOLBAR;
+	}
+	else if (g_strcasecmp (path, "ShowHideStatusbar") == 0) {
+		key = EOG_CONF_UI_STATUSBAR;
+	}
+
+	if (key != NULL) {
+		visible = (g_strcasecmp (state, "1") == 0);
+		gconf_client_set_bool (priv->client, key, visible, NULL);
+	}
+}
+
 static void
 activate_uri_cb (BonoboControlFrame *control_frame, const char *uri, gboolean relative, gpointer data)
 {
@@ -367,6 +411,46 @@ open_uri_list_cleanup (EogWindow *window, GList *txt_uri_list)
 	}
 }
 
+
+static void
+toolbar_visibility_changed_cb (GConfClient *client,
+			       guint        cnxn_id,
+			       GConfEntry  *entry,
+			       gpointer     user_data)
+{
+	EogWindow *window;
+	gboolean visible = TRUE;
+	
+	window = EOG_WINDOW (user_data);
+	
+	if (entry->value != NULL && entry->value->type == GCONF_VALUE_BOOL) {
+		visible = gconf_value_get_bool (entry->value);
+	}
+
+	bonobo_ui_component_set_prop (window->priv->ui_comp, "/Toolbar", "hidden", visible ? "0" : "1", NULL);
+}
+
+static void
+statusbar_visibility_changed_cb (GConfClient *client,
+				 guint        cnxn_id,
+				 GConfEntry  *entry,
+				 gpointer     user_data)
+{
+	EogWindow *window;
+	gboolean visible = TRUE;
+	
+	window = EOG_WINDOW (user_data);
+	
+	if (entry->value != NULL && entry->value->type == GCONF_VALUE_BOOL) {
+		visible = gconf_value_get_bool (entry->value);
+	}
+
+	if (visible)
+		gtk_widget_show (GTK_WIDGET (window->priv->statusbar));
+	else
+		gtk_widget_hide (GTK_WIDGET (window->priv->statusbar));
+}
+
 GType
 eog_window_get_type (void) 
 {
@@ -431,7 +515,13 @@ eog_window_destroy (GtkObject *object)
 
 	/* Clean up GConf-related stuff */
 	if (priv->client) {
-		gconf_client_remove_dir (priv->client, "/apps/eog", NULL);
+		gconf_client_notify_remove (priv->client, priv->view_toolbar_id);
+		priv->view_toolbar_id = 0;
+
+		gconf_client_notify_remove (priv->client, priv->view_statusbar_id);
+		priv->view_statusbar_id = 0;
+
+		gconf_client_remove_dir (priv->client, EOG_CONF_DIR, NULL);
 		g_object_unref (G_OBJECT (priv->client));
 		priv->client = NULL;
 	}
@@ -520,7 +610,7 @@ eog_window_init (EogWindow *window)
 
 	priv->client = gconf_client_get_default ();
 
-	gconf_client_add_dir (priv->client, "/apps/eog",
+	gconf_client_add_dir (priv->client, EOG_CONF_DIR,
 			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
 
 	window_list = g_list_prepend (window_list, window);
@@ -638,6 +728,12 @@ static BonoboUIVerb eog_app_verbs[] = {
 	BONOBO_UI_VERB_END
 };
 
+static void
+widget_realized_cb (GtkWidget *widget, gpointer data)
+{
+	adapt_window_size (EOG_WINDOW (data));
+}
+
 
 /**
  * window_construct:
@@ -650,6 +746,7 @@ eog_window_construct (EogWindow *window)
 {
 	EogWindowPrivate *priv;
 	BonoboUIContainer *ui_container;
+	gboolean visible;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (window));
@@ -658,19 +755,8 @@ eog_window_construct (EogWindow *window)
 
 	ui_container = bonobo_window_get_ui_container (BONOBO_WINDOW (window));
 
-#if 0
-	/* FIXME: it would be nice to allow toolbar
-	   configuration/visibility changes from within eog. But this
-	   has some issues at the moment, see bug #114231. Needs
-	   fixing for 2.4.x.
-	*/
-
-	bonobo_ui_engine_config_set_path (
-		bonobo_window_get_ui_engine (BONOBO_WINDOW (window)),
-		"/apps/eog/shell/ui"); 
-#endif
-						   
 	priv->box = GTK_WIDGET (gtk_vbox_new (FALSE, 0));
+
 	gtk_widget_show (priv->box);
 	bonobo_window_set_contents (BONOBO_WINDOW (window), priv->box);
 
@@ -698,6 +784,16 @@ eog_window_construct (EogWindow *window)
 	priv->statusbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
 	gtk_box_pack_end (GTK_BOX (priv->box), GTK_WIDGET (priv->statusbar),
 			  FALSE, FALSE, 0);
+	/* We connect to the realize signal here, because we must know
+	 * when all the child widgets of an eog window have determined
+	 * their size in order to get a working adapt_window_size
+	 * function. It should be sufficient to connect to the
+	 * statusbar here, because it is one of the deepest widgets in
+	 * the widget hierarchy tree.  FIXME: if we allow hideable
+	 * status-/toolbars then this must be reworked.
+	 */
+	g_signal_connect_after (G_OBJECT (priv->statusbar),
+				"realize", G_CALLBACK (widget_realized_cb), window);
 	gtk_widget_show (GTK_WIDGET (priv->statusbar));
 
 	/* add control frame interface */
@@ -710,6 +806,37 @@ eog_window_construct (EogWindow *window)
 				     DEFAULT_WINDOW_WIDTH,
 				     DEFAULT_WINDOW_HEIGHT);
 	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
+
+	/* show/hide toolbar? */
+	visible = gconf_client_get_bool (priv->client, EOG_CONF_UI_TOOLBAR, NULL);
+	bonobo_ui_component_set_prop (priv->ui_comp, "/commands/ShowHideToolbar", "state",
+				      visible ? "1" : "0", NULL);
+	bonobo_ui_component_set_prop (priv->ui_comp, "/Toolbar", "hidden", visible ? "0" : "1", NULL);
+	bonobo_ui_component_add_listener (priv->ui_comp, "ShowHideToolbar",
+					  (BonoboUIListenerFn) verb_ShowHideAnyBar_cb,
+					  (gpointer) window);
+	priv->view_toolbar_id =
+		gconf_client_notify_add (priv->client,
+					 EOG_CONF_UI_TOOLBAR,
+					 toolbar_visibility_changed_cb,
+					 window, NULL, NULL);
+
+
+	/* show/hide statusbar? */
+	visible = gconf_client_get_bool (priv->client, EOG_CONF_UI_STATUSBAR, NULL);
+	bonobo_ui_component_set_prop (priv->ui_comp, "/commands/ShowHideStatusbar", "state",
+				      visible ? "1" : "0", NULL);
+	if (!visible) {
+		gtk_widget_hide (GTK_WIDGET (priv->statusbar));
+	}
+	bonobo_ui_component_add_listener (priv->ui_comp, "ShowHideStatusbar",
+					  (BonoboUIListenerFn) verb_ShowHideAnyBar_cb,
+					  (gpointer) window);
+	priv->view_statusbar_id =
+		gconf_client_notify_add (priv->client,
+					 EOG_CONF_UI_STATUSBAR,
+					 statusbar_visibility_changed_cb,
+					 window, NULL, NULL);
 }
 
 /**
@@ -751,46 +878,47 @@ eog_window_close (EogWindow *window)
 		bonobo_main_quit ();
 }
 
-
 static void
-adapt_window_size (EogWindow *window, int width, int height)
+adapt_window_size (EogWindow *window)
 {
 	int xthick, ythick;
 	int req_height, req_width;
+	int sb_height;
 	EogWindowPrivate *priv;
+	gboolean sb_visible;
 
 	priv = window->priv;
 
-	/* this is the size of the frame around the vbox */
-	xthick = priv->box->style->xthickness;
-	ythick = priv->box->style->ythickness;
-	req_width = req_height = -1;
+	/* check if the statusbar is visible */
+	g_object_get (G_OBJECT (priv->statusbar), "visible", &sb_visible, NULL);
+
+	if ((priv->desired_width > 0) && (priv->desired_height > 0) &&
+	    (!sb_visible || GTK_WIDGET_REALIZED (priv->statusbar)) &&
+	    GTK_WIDGET_REALIZED (priv->box) &&
+	    GTK_WIDGET_REALIZED (GTK_WIDGET (window)))
+	{
+		/* this is the size of the frame around the vbox */
+		xthick = priv->box->style->xthickness;
+		ythick = priv->box->style->ythickness;
+		req_width = req_height = -1;
+		
+		if (sb_visible) {
+			sb_height = priv->statusbar->allocation.height;
+		}
+		else {
+			sb_height = 0;
+		}
 
-	if (height > 0) {
 		req_height = 
-			height + 
+			priv->desired_height + 
 			(GTK_WIDGET(window)->allocation.height - priv->box->allocation.height) +
-			priv->statusbar->allocation.height + 
-			2 * ythick;
-	}
-	else {
-		req_height = priv->desired_height;
-	}
-	
-	if (width > 0) {
+		        sb_height + 2 * ythick;
+		
 		req_width = 
-			width + 
+			priv->desired_width + 
 			(GTK_WIDGET(window)->allocation.width - priv->box->allocation.width) +
 			2 * xthick;
-	}
-	else {
-		req_width = priv->desired_width;
-	}
 
-	priv->desired_width = req_width;
-	priv->desired_height = req_height;
-
-	if (req_width > 0 && req_height > 0) {
 		gtk_window_resize (GTK_WINDOW (window), req_width, req_height);
 	}
 }
@@ -803,8 +931,10 @@ property_changed_cb (BonoboListener    *listener,
 		     gpointer           user_data)
 {
 	EogWindow *window;
+	EogWindowPrivate *priv;
 
 	window = EOG_WINDOW (user_data);
+	priv = window->priv;
 
 	if (any == NULL) return;
 
@@ -828,10 +958,12 @@ property_changed_cb (BonoboListener    *listener,
 					 BONOBO_ARG_GET_STRING (any));	
 	}
 	else if (!g_ascii_strcasecmp (event_name, PROPERTY_WINDOW_WIDTH)) {
-		adapt_window_size (window, BONOBO_ARG_GET_INT (any), -1);
+		priv->desired_width = BONOBO_ARG_GET_INT (any);
+		adapt_window_size (window);
 	}
 	else if (!g_ascii_strcasecmp (event_name, PROPERTY_WINDOW_HEIGHT)) {
-		adapt_window_size (window, -1, BONOBO_ARG_GET_INT (any));
+		priv->desired_height = BONOBO_ARG_GET_INT (any);
+		adapt_window_size (window);
 	}
 }
 
@@ -885,7 +1017,7 @@ check_for_control_properties (EogWindow *window)
 								 PROPERTY_WINDOW_WIDTH, &ev, window);
 
 			/* query window size */
-			width = bonobo_pbclient_get_long (pb, PROPERTY_WINDOW_WIDTH, &ev);
+			priv->desired_width = bonobo_pbclient_get_long (pb, PROPERTY_WINDOW_WIDTH, &ev);
 		}
 		else if (g_ascii_strcasecmp ((char*) it->data, PROPERTY_WINDOW_HEIGHT) == 0) {
 			bonobo_event_source_client_add_listener (pb, 
@@ -893,7 +1025,7 @@ check_for_control_properties (EogWindow *window)
 								 PROPERTY_WINDOW_HEIGHT, &ev, window);
 
 			/* query window size */
-			height = bonobo_pbclient_get_long (pb, PROPERTY_WINDOW_HEIGHT, &ev);
+			priv->desired_height = bonobo_pbclient_get_long (pb, PROPERTY_WINDOW_HEIGHT, &ev);
 		}
 		else if (g_ascii_strcasecmp ((char*) it->data, PROPERTY_IMAGE_PROGRESS) == 0) {
 			bonobo_event_source_client_add_listener (pb, 
@@ -902,7 +1034,7 @@ check_for_control_properties (EogWindow *window)
 		}
 	}
 
-	adapt_window_size (window, width, height);
+	adapt_window_size (window);
 
 	bonobo_object_release_unref (pb, &ev);
 	CORBA_exception_free (&ev);
@@ -976,14 +1108,12 @@ eog_window_open_list (EogWindow *window, const char *iid, GList *text_uri_list,
 
 	/* remove previously loaded control */
 	if (priv->ctrl_frame != NULL) {
-		gtk_container_remove (GTK_CONTAINER (priv->box), 
-				      GTK_WIDGET (priv->ctrl_widget));
+		bonobo_control_frame_control_deactivate (priv->ctrl_frame);
 		bonobo_object_unref (priv->ctrl_frame);
 		gtk_widget_destroy (priv->ctrl_widget);
 		priv->ctrl_widget = NULL;
 		priv->ctrl_frame = NULL;
 	}
-	g_assert (priv->ctrl_frame == NULL);
 
 	/* create control frame */
 	ui_container = bonobo_window_get_ui_container (BONOBO_WINDOW (window));
